%Mattia Moroder 03.07.2018
%MOLECULAR ORBITALS OF H2O CALCULATED BY LCAO (Linear Combination of
%Atomic Orbitals)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Calculation of the 7 eigenvectors (coefficients of LCAO) and the corresponding 7 eigenenergies 

%cartesian coordinates are used
%used orbitals: 1s_H1, 1s_H2, 1s_0, 2s_0, 2px_0, 2py_0, 2pz_0
%xi,yi,zi are the coordinates of the i-th atom

a_0 =  5.2917721067e-11; %Bohr radius
el = 1.602176634e-19;    %elementary charge
eps = 8.85418781762e-12; % dielectric constant
minval = 10^-13;         % interval used to cut out divergences from integrals
lim = 20*a_0;            %limit for integrals
Z_O = 8;
Z_O_eff1s = 7.658;       %effective charges for oxigen, Wikipedia
Z_O_eff2s = 4.492;
Z_O_eff2p = 4.453;


%define the 5 different orbitals
r = @(x,y,z,xi,yi,zi) sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2);
s_1 = @(x,y,z,xi,yi,zi,Z) sqrt(Z^3/(pi*a_0^3))*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/a_0);  
s_2 = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^3/(2*pi*a_0^3))*(2-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/a_0).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0)); 
p_2x = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^5/(pi*a_0^5))*(x-xi).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0))*1/sqrt(2); %multiplied with 1/sqrt2
p_2y = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^5/(pi*a_0^5))*(y-yi).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0))*1/sqrt(2); %multiplied with 1/sqrt2
p_2z = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^5/(pi*a_0^5))*(z-zi).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0))*1/sqrt(2); %multiplied with 1/sqrt2

%position of the atoms, from jmol
pos_shift = 0;
pos_H1 = pos_shift+[7.6437e-11, 5.9719e-11, 0];
pos_H2 = pos_shift+[-7.6437e-11, 5.9719e-11, 0];
pos_O  = pos_shift+ [0,0,0];

%define all 7 orbitals
s_1_H1 = @(x,y,z) s_1(x,y,z,pos_H1(1),pos_H1(2),pos_H1(3),1);
s_1_H2 = @(x,y,z) s_1(x,y,z,pos_H2(1),pos_H2(2),pos_H1(3),1);
s_1_O = @(x,y,z)  s_1(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff1s);
s_2_O = @(x,y,z) s_2(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2s);
p_2x_O = @(x,y,z) p_2x(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2p);
p_2y_O = @(x,y,z) p_2y(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2p);
p_2z_O = @(x,y,z) p_2z(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2p);

%define cell-array with the 7 orbitals
psi = {s_1_H1,s_1_H2,s_1_O,s_2_O,p_2x_O,p_2y_O,p_2z_O};

%Calculate overlap matrix S
S = zeros(7);
for l = 1:7
    for m = 1:7
        f = @(x,y,z) psi{l}(x,y,z).*psi{m}(x,y,z);
        S(l,m) =integral3(f,-lim,lim,-lim,lim,-lim,-minval) + integral3(f,-lim,lim,-lim,lim,minval,lim);
    end
end

S 

%energies Ei of hydrogen-like atoms with total charge,
% needed vor H matrix calculation
%alternatively use fine structure formula (Wikipedia) or actually calculate
%the Lalplacian
En_H_1 = @(Z) -13.6*el*Z;
En_H_2 = @(Z) -13.6*el*Z/4;
En_O_1 = @(Z) En_H_1(Z).*Z^2;  
En_O_2 = @(Z) En_H_2(Z).*Z^2; 

En_hyd = [En_H_1(1),En_H_2(1),En_O_1(Z_O_eff1s),En_O_2(Z_O_eff2s),En_O_2(Z_O_eff2p),En_O_2(Z_O_eff2p),En_O_2(Z_O_eff2p)];
eff_charge = [1,1,Z_O_eff1s,Z_O_eff2s,Z_O_eff2p,Z_O_eff2p,Z_O_eff2p];

%Hamiltonian-Potential terms
k = -el^2/(4*pi*eps);
Ham_1 = @(x,y,z,Z) Z*k*abs(sqrt((x-pos_H1(1)).^2+(y-pos_H1(2)).^2+(z-pos_H1(3)).^2)).^-1.*(x<pos_H1(1)-minval)+Z*k*abs(sqrt((x-pos_H1(1)).^2+(y-pos_H1(2)).^2+(z-pos_H1(3)).^2)).^-1.*(x>pos_H1(1)+minval)+Z*k*abs(sqrt((x-pos_H1(1)).^2+(y-pos_H1(2)).^2+(z-pos_H1(3)).^2)).^-1.*(y<pos_H1(2)-minval)+Z*k*abs(sqrt((x-pos_H1(1)).^2+(y-pos_H1(2)).^2+(z-pos_H1(3)).^2)).^-1.*(y>pos_H1(2)+minval)+Z*k*abs(sqrt((x-pos_H1(1)).^2+(y-pos_H1(2)).^2+(z-pos_H1(3)).^2)).^-1.*(z<pos_H1(3)-minval)+Z*k*abs(sqrt((x-pos_H1(1)).^2+(y-pos_H1(2)).^2+(z-pos_H1(3)).^2)).^-1.*(z>pos_H1(3)+minval);
Ham_2 = @(x,y,z,Z) Z*k*abs(sqrt((x-pos_H2(1)).^2+(y-pos_H2(2)).^2+(z-pos_H2(3)).^2)).^-1.*(x<pos_H2(1)-minval)+Z*k*abs(sqrt((x-pos_H2(1)).^2+(y-pos_H2(2)).^2+(z-pos_H2(3)).^2)).^-1.*(x>pos_H2(1)+minval)+Z*k*abs(sqrt((x-pos_H2(1)).^2+(y-pos_H2(2)).^2+(z-pos_H2(3)).^2)).^-1.*(y<pos_H2(2)-minval)+Z*k*abs(sqrt((x-pos_H2(1)).^2+(y-pos_H2(2)).^2+(z-pos_H2(3)).^2)).^-1.*(y>pos_H2(2)+minval)+Z*k*abs(sqrt((x-pos_H2(1)).^2+(y-pos_H2(2)).^2+(z-pos_H2(3)).^2)).^-1.*(z<pos_H2(3)-minval)+Z*k*abs(sqrt((x-pos_H2(1)).^2+(y-pos_H2(2)).^2+(z-pos_H2(3)).^2)).^-1.*(z>pos_H2(3)+minval);
Ham_3 = @(x,y,z,Z) Z*k*abs(sqrt(x.^2+y.^2+z.^2)).^-1.*(x & y & z < -minval) + Z*k*abs(sqrt(x.^2+y.^2+z.^2)).^-1.*(x & y & z > minval);
Ham = {Ham_1,Ham_2,Ham_3};

%Calculate H matrix
H = zeros(7);
for i = 1:7
    for j = 1:7
        f1 =@(x,y,z) En_hyd(j)*psi{i}(x,y,z).*psi{j}(x,y,z);
        if j ==1
            fHam = @(x,y,z,Z) Ham_2(x,y,z,1)+Ham_3(x,y,z,Z_O_eff2p);
        elseif j ==2 
            fHam =  @(x,y,z,Z) Ham_1(x,y,z,1)+Ham_3(x,y,z,Z_O_eff2p);
        else
            fHam =  @(x,y,z,Z) Ham_1(x,y,z,1)+Ham_2(x,y,z,1);
        end
           f2 =@(x,y,z) fHam(x,y,z).*psi{i}(x,y,z).*psi{j}(x,y,z); 
           H(i,j) = En_hyd(j)*S(i,j)+ integral3(f2,-lim,lim,-lim,lim,-lim,lim);% integral3(f2,-lim,lim,-lim,lim,minval,lim);
    end
end
H 
%V: eigenvectors, W:eigenvalues
W = S\H
[V,D] = eig(W) 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PLOT OF THE 7 MOLECULAR ORBITALS

%Redefine orbitals with Bohr radius = 1 for plotting
a_0 = 1; % Bohr radius
%define the 5 different orbitals with a_0 = 1

s_1 = @(x,y,z,xi,yi,zi,Z) sqrt(Z^3/(pi*a_0^3))*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/a_0);  
s_2 = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^3/(2*pi*a_0^3))*(2-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/a_0).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0)); 
p_2x = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^5/(pi*a_0^5))*(x-xi).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0))*1/sqrt(2); %multiplied with 1/sqrt2
p_2y = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^5/(pi*a_0^5))*(y-yi).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0))*1/sqrt(2); %multiplied with 1/sqrt2
p_2z = @(x,y,z,xi,yi,zi,Z) 1/4*sqrt(Z^5/(pi*a_0^5))*(z-zi).*exp(-Z*sqrt((x-xi).^2+(y-yi).^2+(z-zi).^2)/(2*a_0))*1/sqrt(2); %multiplied with 1/sqrt2

%position of the atoms divided by Bohr radius
a_0_small = 5.2917721067e-11;
pos_shift = 0;
pos_H1 = pos_shift+[7.6437e-11, 5.9719e-11, 0]/a_0_small;
pos_H2 = pos_shift+[-7.6437e-11, 5.9719e-11, 0]/a_0_small;
pos_O  = pos_shift+ [0,0,0];

%define all 7 orbitals  with a_0 = 1
%syms x y z 
s_1_H1 = @(x,y,z) s_1(x,y,z,pos_H1(1),pos_H1(2),pos_H1(3),1);
s_1_H2 = @(x,y,z) s_1(x,y,z,pos_H2(1),pos_H2(2),pos_H1(3),1);
s_1_O = @(x,y,z)  s_1(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff1s);
s_2_O = @(x,y,z) s_2(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2s);
p_2x_O = @(x,y,z) p_2x(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2p);
p_2y_O = @(x,y,z) p_2y(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2p);
p_2z_O = @(x,y,z) p_2z(x,y,z,pos_O(1),pos_O(2),pos_O(3),Z_O_eff2p);

%define cell-array with the 7 orbitals  with a_0 = 1
psi_nat = {s_1_H1,s_1_H2,s_1_O,s_2_O,p_2x_O,p_2y_O,p_2z_O};

%calculate the molecular orbitals
mol_orb = cell(1,7);
for i = 1:7
    temp = @(x,y,z) 0*x+y*0+0*z;
    for j = 1:7
           mol_orb{i} = @(x,y,z)V(j,i)*psi_nat{j}(x,y,z)+temp(x,y,z);
           temp =  @(x,y,z) mol_orb{i}(x,y,z);
    end
end
         
% setting the grid 
border = 20;
accuracy = 400;
probabilitydensity = 1e-6;
raster = linspace(-border, border, accuracy);
[x, y, z] = meshgrid(raster, raster, raster);

%plot atomic orbitals 1s, 2s, 2px, 2py, 2pz of oxygen
titstr ={'1s orbital','2s orbital', '2px orbital','2py orbital','2pz orbital'};
for i = 3:7
    figure
    colors = sign(psi_nat{i}(x,y,z));   
    isosurface(psi_nat{i}(x,y,z).^2, probabilitydensity, colors); 
    colormap([1 0 0; 0 1 0])
    material dull
    set(gcf,'color', 'w');
    set(gca,'CameraViewAngle', 45, 'Projection', 'perspective');
    camzoom(4)
    axis equal
    axis vis3d
    xlabel('$x$', 'interpreter', 'latex', 'FontSize', 20)
    ylabel('$y$', 'interpreter', 'latex', 'FontSize', 20)
    zlabel('$z$', 'interpreter', 'latex', 'FontSize', 20)
    title(titstr{i-2})
end

% plot molecular orbital with wavefunction phase
titstr2 = {'molecular orbital 1','molecular orbital 6 (HOMO)','molecular orbital 7 (LUMO)','molecular orbital 5',...
'molecular orbital 4','molecular orbital 2','molecular orbital 3'};
for i = 1:7
    figure
    colors = sign(mol_orb{i}(x,y,z));   
    isosurface(mol_orb{i}(x,y,z).^2, probabilitydensity, colors); 
    colormap([1 1 0;0.747 0.148 1])
    material dull
    set(gcf,'color', 'w');
    set(gca,'CameraViewAngle', 45, 'Projection', 'perspective');
    camzoom(4)
    axis equal
    axis vis3d
    xlabel('$x$', 'interpreter', 'latex', 'FontSize', 20)
    ylabel('$y$', 'interpreter', 'latex', 'FontSize', 20)
    zlabel('$z$', 'interpreter', 'latex', 'FontSize', 20)
    title(titstr2{i})
end